package main

import "core:math"
import rl "vendor:raylib" 

Vec3 :: [3]f32

Entity :: struct {
	position : Vec3,
	velocity : Vec3,
	size : Vec3,
	update : proc(entity: ^Entity, delta: f32),
}

PAL_INT :: [6]u32 {
	0x272744ff,
	0x494d7eff,
	0x8b6d9cff,
	0xc69fa5ff,
	0xf2d3abff,
	0xfbf5efff,
}

PAL := [6]rl.Color{}

main :: proc() {
	for v, i in PAL_INT {
		PAL[i] = rl.GetColor(v)
	}

	rl.InitWindow(800, 400, "By The Fire")
	rl.SetTargetFPS(60)
	
	img := rl.GenImageChecked(64, 64, 8, 8, PAL[1], PAL[2])
	defer rl.UnloadImage(img)
	tex := rl.LoadTextureFromImage(img)
	defer rl.UnloadTexture(tex)
	rl.SetTextureFilter(tex, rl.TextureFilter.POINT)

	plane := rl.GenMeshPlane(32, 32, 16, 16)
	defer rl.UnloadMesh(plane)
	mat := rl.LoadMaterialDefault()
	rl.SetMaterialTexture(&mat, rl.MaterialMapIndex.ALBEDO, tex)

	player := player_create()

	rl.DisableCursor()

	for !rl.WindowShouldClose() {
		using rl 
		player_update(&player, GetFrameTime())

		BeginDrawing()
		BeginMode3D(player.camera)

			ClearBackground(PAL[0])
			rlPushMatrix()
			rlRotatef(45, 0, 1, 0)
			DrawMesh(plane, mat, rl.MatrixIdentity())
			rlPopMatrix()


			rlPushMatrix()
			rlTranslatef(player.position.x, player.position.y, player.position.z)
			rlTranslatef(player.cameraOffset.x, player.cameraOffset.y, player.cameraOffset.z)
			rlRotatef(math.to_degrees(player.direction), 0, 1, 0)
			rlRotatef(math.to_degrees(-player.yaw), 1, 0, 0)
			rlTranslatef(-1, -0.3, 1)
			rlScalef(0.5, 0.5, 0.5)
			DrawCube({0, 0, 0}, 1,1,1, PAL[4])
			rlTranslatef(0, 0, 1)
			rlScalef(0.4, 0.4, 1)
			DrawCube({0, 0, 0}, 1,1,1, PAL[4])

			rlPopMatrix()

		EndMode3D()
		EndDrawing()
	}
}
