package main

import "core:math"
import rl "vendor:raylib"

Player :: struct {
	using entity: Entity,
	cameraOffset: Vec3,
	direction: f32,
	yaw: f32,
	camera: rl.Camera3D,
}

player_create :: proc() -> Player {
	return Player {
		size = {0.5, 2, 0.5},
		cameraOffset = {0, 1.8, 0},
		camera = rl.Camera3D{
			up = {0, 1, 0},
			fovy = 80,
			target = {0, 0, 10},
			projection = rl.CameraProjection.PERSPECTIVE,
		},
	}
}

player_update :: proc(using player: ^Player, delta: f32) {
	using rl
	mouseDelta := GetMouseDelta()

	direction -= mouseDelta.x * 0.003
	yaw -= mouseDelta.y * 0.003

	yaw = Clamp(yaw, -math.PI/2+0.001, math.PI/2-0.001)
	
	dir_right := direction + math.PI / 2
	forward := Vec3{math.sin(direction), 0, math.cos(direction)}
	right := Vec3{math.sin(dir_right), 0, math.cos(dir_right)}

	motion := Vec3{}

	if IsKeyDown(KeyboardKey.W) {motion += forward * 0.1}
	if IsKeyDown(KeyboardKey.S) {motion -= forward * 0.1}
	if IsKeyDown(KeyboardKey.A) {motion += right * 0.1}
	if IsKeyDown(KeyboardKey.D) {motion -= right * 0.1}
	motion = rl.Vector3ClampValue(motion, 0, 0.2)
	velocity.xz = motion.xz
	position += velocity

	camera.position = position + cameraOffset
	
	target := Vec3{
		math.sin(direction) * math.cos(yaw),
		math.sin(yaw),
		math.cos(direction) * math.cos(yaw),
	}
	camera.target = position + cameraOffset + target
}
